@@ -165,7 +165,7 @@ class ProviderService(base.AppService):
         :returns: a context manager of the provider instance.
         """
         if not project_name:
-            project_name = self._services.get("project").get().name
+            project_name = self._project.name
         instance_name = self._get_instance_name(work_dir, build_info, project_name)
         emit.debug(f"Preparing managed instance {instance_name!r}")
         base_name = bases.BaseName(
@@ -176,6 +176,7 @@ class ProviderService(base.AppService):
         provider = self.get_provider(name=self.__provider_name)
 
         provider.ensure_provider_is_available()
+        shutdown_delay = self._services.get("config").get("idle_mins")
 
         if clean_existing:
             self._clean_instance(provider, work_dir, build_info, project_name)
@@ -189,6 +190,7 @@ class ProviderService(base.AppService):
             allow_unstable=allow_unstable,
             use_base_instance=use_base_instance,
             prepare_instance=prepare_instance,
+            shutdown_delay_mins=shutdown_delay,
         ) as instance:
             instance.mount(
                 host_source=work_dir,
@@ -210,7 +212,7 @@ class ProviderService(base.AppService):
         *,
         instance_name: str,
         **kwargs: bool | str | pathlib.Path | None,
-    ) -> craft_providers.Base:
+    ) -> craft_providers.Base[enum.Enum]:
         """Get the base configuration from a base name.
 
         :param base_name: The base to lookup.
@@ -322,10 +324,8 @@ class ProviderService(base.AppService):
             target = "environments" if len(build_plan) > 1 else "environment"
             emit.progress(f"Cleaning build {target}")
 
-        project_name = self._services.get("project").get().name
-
         for info in build_plan:
-            self._clean_instance(provider, self._work_dir, info, project_name)
+            self._clean_instance(provider, self._work_dir, info, self._project.name)
 
     def _get_instance_name(
         self,
